package com.zhilian.business.controller;

import com.zhilian.business.domain.BreakLog;
import com.zhilian.business.service.BreakLogService;
import com.zhilian.common.core.domain.Result;
import io.swagger.models.auth.In;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @ClassName BreakLogController
 * @Description 故障日志业务实现
 * @Author YuanYongH
 * @Date 2024/4/8 20:48
 */
@RestController
public class BreakLogController {

    @Autowired
    private BreakLogService breakLogService;

    /**
     * @Description //故障日志列表
     * @Date 2024/4/10
     * @return com.zhilian.common.core.domain.Result<java.util.List<com.zhilian.business.domain.BreakLog>>
    **/
    @PostMapping("log")
    public Result<List<BreakLog>> breakLog() {
        List<BreakLog> list = breakLogService.breakLog();
        Result<List<BreakLog>> success = Result.success(list);
        return success;
    }

    /**
     * @Description //故障日志添加
     * @Date 2024/4/10
     * @param breakLog
     * @return com.zhilian.common.core.domain.Result
    **/
    @PostMapping("logAdd")
    public Result logAdd(@RequestBody BreakLog breakLog){
        int i = breakLogService.logAdd(breakLog);
        Result<Integer> success = Result.success(i);
        return success;
    }

    /**
     * @Description // 故障日志修改
     * @Date 2024/4/10
     * @param breakLog
     * @return com.zhilian.common.core.domain.Result
    **/
    @PostMapping("updLog")
    public Result updLog(@RequestBody BreakLog breakLog){
        int i = breakLogService.updLog(breakLog);
        Result<Integer> success = Result.success(i);
        return success;
    }

    @PostMapping("logDel/{breakLogId}")
    public Result logDel(@PathVariable Integer breakLogId){
        int del = breakLogService.logDel(breakLogId);
        Result<Integer> success = Result.success(del);
        return success;
    }

    /**
     *  根据故障码和vin判断数据库中是否有数据
     * @param breakCode
     * @param breakVin
     * @return
     */
    @GetMapping("/checkIfBreakLogExists/{breakCode}/{breakVin}")
    public ResponseEntity <Boolean> checkIfBreakLogExists(@PathVariable("breakCode") String breakCode,@PathVariable("breakVin") String breakVin){
        boolean exists =breakLogService.checkIfBreakLogExists(breakCode,breakVin);
        return ResponseEntity.ok(exists);
    }

}
