package com.zhilian.business.controller;

import java.util.List;
import javax.servlet.http.HttpServletResponse;

import com.zhilian.business.domain.BreakLog;
import com.zhilian.business.domain.request.BreakReq;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.zhilian.common.log.annotation.Log;
import com.zhilian.common.log.enums.BusinessType;
import com.zhilian.common.security.annotation.RequiresPermissions;
import com.zhilian.business.domain.BusinessBreak;
import com.zhilian.business.service.IBusinessBreakService;
import com.zhilian.common.core.web.controller.BaseController;
import com.zhilian.common.core.domain.Result;
import com.zhilian.common.core.utils.poi.ExcelUtil;
import com.zhilian.common.core.web.page.*;

/**
 * 故障Controller
 *
 * @author Yy
 * @date 2024-04-07
 */
@RestController
@RequestMapping("/break")
public class BusinessBreakController extends BaseController{
    @Autowired
    private IBusinessBreakService businessBreakService;

    /**
     * 查询故障列表
     */
//    @RequiresPermissions("business:break:list")
    @GetMapping("/list")
    public Result<TableDataInfo<BusinessBreak>> list(BreakReq breakReq)
    {
        startPage();
        List<BusinessBreak> list = businessBreakService.selectBusinessBreakList(breakReq);
        return getDataTable(list);
    }

    /**
     * 导出故障列表
     */
    @RequiresPermissions("business:break:export")
    @Log(title = "故障", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BreakReq breakReq)
    {
        List<BusinessBreak> list = businessBreakService.selectBusinessBreakList(breakReq);
        ExcelUtil<BusinessBreak> util = new ExcelUtil<BusinessBreak>(BusinessBreak.class);
        util.exportExcel(response, list, "故障数据");
    }

    /**
     * 获取故障详细信息
     */
    @RequiresPermissions("business:break:query")
    @GetMapping(value = "/{breakId}")
    public Result getInfo(@PathVariable("breakId") Long breakId)
    {
        return success(businessBreakService.selectBusinessBreakByBreakId(breakId));
    }

    /**
     * 新增故障
     */
    @RequiresPermissions("business:break:add")
    @Log(title = "故障", businessType = BusinessType.INSERT)
    @PostMapping
    public Result add(@RequestBody BusinessBreak businessBreak)
    {
        return toAjax(businessBreakService.insertBusinessBreak(businessBreak));
    }

    /**
     * 修改故障
     */
    @RequiresPermissions("business:break:edit")
    @Log(title = "故障", businessType = BusinessType.UPDATE)
    @PutMapping
    public Result edit(@RequestBody BusinessBreak businessBreak)
    {
        return toAjax(businessBreakService.updateBusinessBreak(businessBreak));
    }

    /**
     * 删除故障
     */
    @RequiresPermissions("business:break:remove")
    @Log(title = "故障", businessType = BusinessType.DELETE)
	@DeleteMapping("/{breakIds}")
    public Result remove(@PathVariable Long[] breakIds)
    {
        return toAjax(businessBreakService.deleteBusinessBreakByBreakIds(breakIds));
    }

    /**
     * 故障日志
     */
    @PostMapping("breakLog")
    public Result<List<BreakLog>> breakLog(){
        List<BreakLog> list = businessBreakService.breakLog();
        Result<List<BreakLog>> success = Result.success(list);
        return success;
    }
}
