package com.zhilian.business.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.zhilian.business.domain.Markers;
import com.zhilian.business.domain.middle.MarkersFence;
import com.zhilian.business.service.MarkersFenceService;
import com.zhilian.business.service.MarkersService;
import com.zhilian.common.core.domain.Result;
import com.zhilian.common.core.web.controller.BaseController;
import com.zhilian.common.core.web.page.TableDataInfo;
import com.zhilian.common.security.utils.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;
/**
  * @ClassName MarkersController
  * @Description 车辆标识控制层
  * @Author ZeJinG.Su
  * @Date  15:31 2024/4/8
 */
@RestController
@RequestMapping("/markers")
public class MarkersController extends BaseController {
    @Autowired
    private MarkersService markersService;
    @Autowired
    private MarkersFenceService markersFenceService;

    /**
     * 车辆标识列表
     * @param markers
     * @return
     */
    @PostMapping("list")
    public Result<TableDataInfo<Markers>> list(@RequestBody Markers markers) {
        startPage();
        List<Markers> result = markersService.selectMarkersList(markers);
        return getDataTable(result);
    }

    /**
     * 车辆标识全部数据
     * @return
     */
    @GetMapping("markersVehicle")
    public Result<List<Markers>> markersVehicle() {
        List<Markers> listMarkers = markersService.listMarkers();
        return Result.success(listMarkers);
    }

    /**
     * 新增车辆标识
     * @param markers
     * @return
     */
    @PostMapping("insert")
    public Result insert(@RequestBody Markers markers){
        return toAjax(markersService.save(markers));
    }

    /**
     * 修改车辆标识
     * @param markers
     * @return
     */
    @PostMapping("update")
    public Result update(@RequestBody Markers markers) {
        markers.setUpdateBy(SecurityUtils.getUsername());
        markers.setUpdateTime(new Date());
        markersFenceService.removeAllFence(markers);

        //修改原表数据
        boolean update =markersService.updateById(markers);

        //重新添加中间表数据
        markersFenceService.insert(markers);

        return toAjax(update);
    }

    /**
     * 删除车辆标识
     * @param markersId
     * @return
     */
    @PostMapping("delete/{markersId}")
    public Result delete(@PathVariable Long markersId) {
        markersFenceList(markersId).forEach(markersFence -> {
            markersFenceService.removeById(markersId);
        });
        return toAjax(markersService.removeById(markersId));
    }

    /**
     * 绑定电子围栏
     * @param markers
     * @return
     */
    @PostMapping("insertMarkersFence")
    public Result insertMarkersFenceMarkers(@RequestBody Markers markers){
        boolean insert = markersFenceService.insert(markers);
        return toAjax(insert);
    }

    /**
     * 查询标识与电子围栏中间表信息
     * @param markersId
     * @return
     */
    @PostMapping("markersFenceList/{markersId}")
    public List<MarkersFence> markersFenceList(@PathVariable Long markersId){
        LambdaQueryWrapper<MarkersFence> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(MarkersFence::getMarkerId, markersId);
        return markersFenceService.list(queryWrapper);
    }

}
