package com.zhilian.business.service.impl;

import java.util.List;

import com.zhilian.business.domain.BreakLog;
import com.zhilian.business.domain.request.BreakReq;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.zhilian.business.mapper.BusinessBreakMapper;
import com.zhilian.business.domain.BusinessBreak;
import com.zhilian.business.service.IBusinessBreakService;

/**
 * 故障Service业务层处理
 *
 * @author Yy
 * @date 2024-04-07
 */
@Service
public class BusinessBreakServiceImpl implements IBusinessBreakService
{
    @Autowired
    private BusinessBreakMapper businessBreakMapper;

    /**
     * 查询故障
     *
     * @param breakId 故障主键
     * @return 故障
     */
    @Override
    public BusinessBreak selectBusinessBreakByBreakId(Long breakId)
    {
        return businessBreakMapper.selectBusinessBreakByBreakId(breakId);
    }

    /**
     * 查询故障列表
     *
     * @param breakReq 故障
     * @return 故障
     */
    @Override
    public List<BusinessBreak> selectBusinessBreakList(BreakReq breakReq)
    {
        return businessBreakMapper.selectBusinessBreakList(breakReq);
    }

    /**
     * 新增故障
     *
     * @param businessBreak 故障
     * @return 结果
     */
    @Override
    public int insertBusinessBreak(BusinessBreak businessBreak)
    {
        return businessBreakMapper.insertBusinessBreak(businessBreak);
    }

    /**
     * 修改故障
     *
     * @param businessBreak 故障
     * @return 结果
     */
    @Override
    public int updateBusinessBreak(BusinessBreak businessBreak)
    {
        return businessBreakMapper.updateBusinessBreak(businessBreak);
    }

    /**
     * 批量删除故障
     *
     * @param breakIds 需要删除的故障主键
     * @return 结果
     */
    @Override
    public int deleteBusinessBreakByBreakIds(Long[] breakIds)
    {
        return businessBreakMapper.deleteBusinessBreakByBreakIds(breakIds);
    }

    /**
     * 删除故障信息
     *
     * @param breakId 故障主键
     * @return 结果
     */
    @Override
    public int deleteBusinessBreakByBreakId(Long breakId)
    {
        return businessBreakMapper.deleteBusinessBreakByBreakId(breakId);
    }

    @Override
    public List<BreakLog> breakLog() {
        return businessBreakMapper.breakLog();
    }
}
