package com.zhilian.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zhilian.business.domain.Fence;
import com.zhilian.business.mapper.FenceMapper;
import com.zhilian.business.service.FenceService;
import com.zhilian.common.core.utils.StringUtils;
import com.zhilian.common.redis.service.RedisService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
public class FenceServiceImpl extends ServiceImpl<FenceMapper, Fence> implements FenceService {
    @Autowired
    private RedisService redisService;

    private static final String FENCE_INFO_KEY = "fence_info:";
    //电子围栏信息查询
    @Override
    public List<Fence> fenceList(Fence fence) {
        if (fence.getFenceName() == null && fence.getFenceTypeId() == null && fence.getFenceState() == null) {
            return this.list();
        }

        LambdaQueryWrapper<Fence> queryWrapper = new LambdaQueryWrapper<>();

        // 模糊条件 - 电子围栏名称
        if (StringUtils.isNotEmpty(fence.getFenceName())) {
            queryWrapper.like(Fence::getFenceName, fence.getFenceName());
        }

        // 精确查询 - 查询类型
        if (fence.getFenceTypeId() != null) {
            queryWrapper.eq(Fence::getFenceTypeId, fence.getFenceTypeId());
        }

        // 模糊查询 - 状态
        if (fence.getFenceState() != null) {
            queryWrapper.eq(Fence::getFenceState, fence.getFenceState());
        }
        if(!redisService.hasKey(FENCE_INFO_KEY)){
            fenceRedisData();
        }
        return this.list(queryWrapper);
    }

    //缓存电子围栏信息
    @Override
    public void fenceRedisData() {
        List<Fence> list = this.list();
        for (Fence fence : list) {
            redisService.setCacheSet(FENCE_INFO_KEY+fence.getFenceId(),fence.getFenceMessage());
        }
    }
    //删除电子围栏全部数据
    private void fenceRedisDelete(){
        List<Fence> list = this.list();
        for (Fence fence : list) {
            redisService.deleteObject(FENCE_INFO_KEY+fence.getFenceId());
        }
    }
    //添加电子围栏信息
    @Override
    public boolean saveFence(Fence fence) {
        boolean save = this.save(fence);
        return save;
    }
    //绑定经纬度地图
    @Override
    public boolean updateByMap(Fence fence) {
        boolean updateByMap = this.updateById(fence);
        if(redisService.hasKey(FENCE_INFO_KEY+fence.getFenceId())){
            redisService.deleteObject(FENCE_INFO_KEY+fence.getFenceId());
        }
        redisService.setCacheSet(FENCE_INFO_KEY+fence.getFenceId(),fence.getFenceMessage());
        return updateByMap;
    }


    //删除电子围栏
    @Override
    public boolean removeByFence(String fenceId) {
        boolean remove = this.removeById(Integer.parseInt(fenceId));
        if(redisService.hasKey(FENCE_INFO_KEY+fenceId)){
            redisService.deleteObject(FENCE_INFO_KEY+fenceId);
        }
        return remove;
    }


}
