package com.zhilian.resolver.model;
import com.zhilian.common.core.utils.SpringUtils;
import com.zhilian.common.redis.service.RedisService;
import com.zhilian.common.resolver.domain.ResolverReportData;
import com.zhilian.resolver.service.ResolverEventService;
import lombok.extern.slf4j.Slf4j;
import org.apache.kafka.clients.consumer.ConsumerConfig;
import org.apache.kafka.clients.consumer.ConsumerRecords;
import org.apache.kafka.clients.consumer.KafkaConsumer;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.time.Duration;
import java.util.*;
import java.util.stream.Collectors;

import static com.zhilian.resolver.utils.ConvertUtils.hexStringToString;
import static com.zhilian.resolver.utils.ConvertUtils.parseVehicleData;

/**
  * @ClassName ModelsKafkaMessage
  * @Description 描述
  * @Author Can.J
  * @Date  2024/4/8
 */
@Component
@Slf4j
public class ModelsKafkaMessage {
    @Autowired
    private RedisService redisService;
    private static final String TOPIC_NAME = "vehicle-topic";
    private static final String BOOTSTRAP_SERVERS = "10.10.25.5:9092";



    /**
     * 消费者配置
     * @return
     */
    @PostConstruct
    private void consumerMessages() {
        Thread kafkaConsumerThread = new Thread(() -> {
        log.info("启动线程");
        Properties props = new Properties();
        props.put(ConsumerConfig.BOOTSTRAP_SERVERS_CONFIG, BOOTSTRAP_SERVERS);
        props.put(ConsumerConfig.GROUP_ID_CONFIG, "my-consumer-group");
        props.put(ConsumerConfig.KEY_DESERIALIZER_CLASS_CONFIG, "org.apache.kafka.common.serialization.StringDeserializer");
        props.put(ConsumerConfig.VALUE_DESERIALIZER_CLASS_CONFIG, "org.apache.kafka.common.serialization.StringDeserializer");

        //创建消费者
        KafkaConsumer<String, String> consumer = new KafkaConsumer<>(props);

        try {

            //订阅主题
            consumer.subscribe(Collections.singletonList(TOPIC_NAME));

            //持续消费消息
            while (true) {
                ConsumerRecords<String, String> records = consumer.poll(Duration.ofMillis(100));
                records.forEach(record -> {
                    System.out.println("接收到的数据：" + record.value());
                    String str = hexStringToString(record.value());
                    List<ResolverReportData> resolverReportDataList = parseVehicleData(str);
                    for (ResolverReportData vehicleData : resolverReportDataList) {
                        log.info("解析到车辆数据：{}", vehicleData);

                        //获取vin
                        String vin = vehicleData.getVin();
                        //获取事件集
                        Set<Object> cacheSet = redisService.getCacheSet("vehicle-event:" + vin);
                        List<String> events = cacheSet.stream().map(item -> {
                            return String.valueOf(item);
                        }).collect(Collectors.toList());
                        log.info("事件集合：{}",events);

                            log.info("解析到车辆数据：{}", vehicleData);
                            for (String stringEvent : events) {
                                ResolverEventService resolverEventService =SpringUtils.getBean(stringEvent);
                                resolverEventService.execute(vehicleData);
                        }
                    }
                });
            }
        } catch (Exception e) {
            log.error("Error occurred in Kafka consumer thread", e);
        } finally {
            consumer.close();
        }
        });
        kafkaConsumerThread.start();
    }


}
