package com.zhilian.resolver.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.zhilian.common.core.domain.Result;
import com.zhilian.common.core.utils.StringUtils;
import com.zhilian.common.core.web.page.PageResult;
import com.zhilian.common.resolver.domain.req.ReportByEsReq;
import com.zhilian.common.resolver.domain.resp.ReportByEsResp;
import com.zhilian.resolver.service.ReportByEsService;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.common.text.Text;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.SearchHits;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.elasticsearch.search.fetch.subphase.highlight.HighlightBuilder;
import org.elasticsearch.search.fetch.subphase.highlight.HighlightField;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Map;

/**
  * @ClassName ReportByEsServiceImpl
  * @Description ES 查询报文业务实现层  不使用此业务
  * @Author Can.J
  * @Date  2024/3/28 0:03
 */
@Service
public class ReportByEsServiceImpl implements ReportByEsService {
    /**
      * @Description RestHighLevelClient
      * @Author Can.J
      * @Date  2024/3/28 22:10
     */
    @Autowired
    private RestHighLevelClient restHighLevelClient;

    public static final String INDEX_REPORT = "report";
    @Override
    public Result<PageResult<ReportByEsResp>> queryReportByEs(ReportByEsReq reportByEsReq) {
        long total = 0;
        ArrayList<ReportByEsResp> list = new ArrayList<>();
        try {
            SearchRequest searchRequest = new SearchRequest(INDEX_REPORT);
            SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
            BoolQueryBuilder boolQuery = QueryBuilders.boolQuery();
            if (StringUtils.isNotEmpty(reportByEsReq.getVin())) {
                boolQuery.must(QueryBuilders.matchQuery("vin", reportByEsReq.getVin()));
            }

            searchSourceBuilder.query(boolQuery);
            searchSourceBuilder.from((reportByEsReq.getPageNum() - 1) * reportByEsReq.getPageSize());
            searchSourceBuilder.size(reportByEsReq.getPageSize());

            searchSourceBuilder.highlighter(new HighlightBuilder()
                    .field("vin")
                    .preTags("<font color='red'>")
                    .postTags("</font>"));
            searchRequest.source(searchSourceBuilder);
            SearchResponse searchResponse = restHighLevelClient.search(searchRequest, RequestOptions.DEFAULT);
            SearchHits searchHits = searchResponse.getHits();
            total = searchHits.getTotalHits().value;

            SearchHit[] hits = searchHits.getHits();
            for (SearchHit hit : hits) {
                String sourceAsString = hit.getSourceAsString();
                ReportByEsResp reportByEsResp = JSONObject.parseObject(sourceAsString, ReportByEsResp.class);
                reportByEsResp.setReportId(Long.parseLong(hit.getId()));

                Map<String, HighlightField> highlightFields = hit.getHighlightFields();
                if (highlightFields != null) {
                    HighlightField highlightField = highlightFields.get("vin");
                    if (highlightField != null) {
                        StringBuilder sb = new StringBuilder();
                        for (Text fragment : highlightField.getFragments()) {
                            sb.append(fragment);
                            System.out.println("打印" + fragment);
                        }
                        reportByEsResp.setVin(sb.toString());
                    }
                }
                list.add(reportByEsResp);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return PageResult.toResult(total, list);
    }
}
