package com.zhilian.system.controller;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.zhilian.common.log.annotation.Log;
import com.zhilian.common.log.enums.BusinessType;
import com.zhilian.common.security.annotation.RequiresPermissions;
import com.zhilian.system.domain.BusinessBreak;
import com.zhilian.system.service.IBusinessBreakService;
import com.zhilian.common.core.web.controller.BaseController;
import com.zhilian.common.core.domain.Result;
import com.zhilian.common.core.utils.poi.ExcelUtil;
import com.zhilian.common.core.web.page.TableDataInfo;

/**
 *
 * @author y
 * @date 2024-04-07
 */
@RestController
@RequestMapping("/break")
public class BusinessBreakController extends BaseController
{
    @Autowired
    private IBusinessBreakService businessBreakService;

    /**
     * 查询列表
     */
    @RequiresPermissions("system:break:list")
    @GetMapping("/list")
    public Result<TableDataInfo<BusinessBreak>> list(BusinessBreak businessBreak)
    {
        startPage();
        List<BusinessBreak> list = businessBreakService.selectBusinessBreakList(businessBreak);
        return getDataTable(list);
    }

    /**
     * 导出列表
     */
    @RequiresPermissions("system:break:export")
    @Log(title = "", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BusinessBreak businessBreak)
    {
        List<BusinessBreak> list = businessBreakService.selectBusinessBreakList(businessBreak);
        ExcelUtil<BusinessBreak> util = new ExcelUtil<BusinessBreak>(BusinessBreak.class);
        util.exportExcel(response, list, "故障码数据");
    }

    /**
     * 获取详细信息
     */
    @RequiresPermissions("system:break:query")
    @GetMapping(value = "/{breakId}")
    public Result getInfo(@PathVariable("breakId") Long breakId)
    {
        return success(businessBreakService.selectBusinessBreakByBreakId(breakId));
    }

    /**
     * 新增
     */
    @RequiresPermissions("system:break:add")
    @Log(title = "", businessType = BusinessType.INSERT)
    @PostMapping
    public Result add(@RequestBody BusinessBreak businessBreak)
    {
        return toAjax(businessBreakService.insertBusinessBreak(businessBreak));
    }

    /**
     * 修改
     */
    @RequiresPermissions("system:break:edit")
    @Log(title = "", businessType = BusinessType.UPDATE)
    @PutMapping
    public Result edit(@RequestBody BusinessBreak businessBreak)
    {
        return toAjax(businessBreakService.updateBusinessBreak(businessBreak));
    }

    /**
     * 删除
     */
    @RequiresPermissions("system:break:remove")
    @Log(title = "", businessType = BusinessType.DELETE)
	@DeleteMapping("/{breakIds}")
    public Result remove(@PathVariable Long[] breakIds)
    {
        return toAjax(businessBreakService.deleteBusinessBreakByBreakIds(breakIds));
    }
}
